# Generated by Django 4.2.13 on 2024-06-24 18:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0215_remove_enterprisecustomer_career_engagement_network_message_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisegroupmembership',
            name='errored_at',
            field=models.DateTimeField(blank=True, default=None, help_text='The last time the membership action was in an error state. Null means the membership is not errored.', null=True),
        ),
        migrations.AddField(
            model_name='historicalenterprisegroupmembership',
            name='errored_at',
            field=models.DateTimeField(blank=True, default=None, help_text='The last time the membership action was in an error state. Null means the membership is not errored.', null=True),
        ),
        migrations.AlterField(
            model_name='enterprisegroupmembership',
            name='status',
            field=models.CharField(blank=True, choices=[('removed', 'Removed'), ('accepted', 'Accepted'), ('pending', 'Pending'), ('internal_api_error', 'Internal API error'), ('email_error', 'Email error')], default='pending', help_text='Current status of the membership record', max_length=20, null=True, verbose_name='Membership Status'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisegroupmembership',
            name='status',
            field=models.CharField(blank=True, choices=[('removed', 'Removed'), ('accepted', 'Accepted'), ('pending', 'Pending'), ('internal_api_error', 'Internal API error'), ('email_error', 'Email error')], default='pending', help_text='Current status of the membership record', max_length=20, null=True, verbose_name='Membership Status'),
        ),
    ]
