# Generated by Django 4.2.13 on 2024-06-12 07:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0210_enterprisecustomer_enable_learner_portal_sidebar_message_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomer',
            name='show_videos_in_learner_portal_search_results',
            field=models.BooleanField(default=False, help_text='If checked, videos will be displayed in the search results on the learner portal.', verbose_name='Show videos in learner portal search results'),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomer',
            name='show_videos_in_learner_portal_search_results',
            field=models.BooleanField(default=False, help_text='If checked, videos will be displayed in the search results on the learner portal.', verbose_name='Show videos in learner portal search results'),
        ),
    ]
