# Generated by Django 3.2.20 on 2023-08-08 09:23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cornerstone', '0028_auto_20230719_1621'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='historicalcornerstoneenterprisecustomerconfiguration',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical cornerstone enterprise customer configuration', 'verbose_name_plural': 'historical cornerstone enterprise customer configurations'},
        ),
    ]
