# Generated by Django 4.2.16 on 2024-10-23 22:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0224_alter_enterprisegroup_applies_to_all_contexts_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='enterprisegroup',
            name='applies_to_all_contexts',
        ),
        migrations.RemoveField(
            model_name='historicalenterprisegroup',
            name='applies_to_all_contexts',
        ),
        migrations.AlterField(
            model_name='enterprisegroup',
            name='name',
            field=models.CharField(help_text='Specifies enterprise group name.', max_length=250),
        ),
        migrations.AlterField(
            model_name='historicalenterprisegroup',
            name='name',
            field=models.CharField(help_text='Specifies enterprise group name.', max_length=250),
        ),
    ]
