# Generated by Django 3.2.23 on 2024-04-23 10:57

from django.db import migrations
from integrated_channels.blackboard.utils import populate_decrypted_fields_blackboard


class Migration(migrations.Migration):

    dependencies = [
        ('blackboard', '0020_auto_20240422_1709'),
    ]

    operations = [
        migrations.RunPython(populate_decrypted_fields_blackboard, reverse_code=migrations.RunPython.noop),
    ]
