# Generated by Django 4.2.15 on 2024-09-02 09:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('canvas', '0038_alter_canvasenterprisecustomerconfiguration_client_id_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='canvasenterprisecustomerconfiguration',
            name='client_id',
        ),
        migrations.RemoveField(
            model_name='canvasenterprisecustomerconfiguration',
            name='client_secret',
        ),
    ]
