# Generated by Django 4.2.10 on 2024-06-13 10:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0211_enterprisecustomer_show_videos_in_learner_portal_search_results_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='career_engagement_network_message',
            field=models.TextField(blank=True, help_text='Message text shown on the learner portal dashboard for career engagement network.', null=True),
        ),
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='learner_portal_sidebar_content',
            field=models.TextField(blank=True, help_text='Text shown on the learner portal dashboard for customer specific purposes. Open HTML field.', null=True),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='career_engagement_network_message',
            field=models.TextField(blank=True, help_text='Message text shown on the learner portal dashboard for career engagement network.', null=True),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='learner_portal_sidebar_content',
            field=models.TextField(blank=True, help_text='Text shown on the learner portal dashboard for customer specific purposes. Open HTML field.', null=True),
        ),
    ]
